@@ -30,6 +30,7 @@ import math
 import os
 import pickle
 
+from absl import logging
 import numpy as np
 import tensorflow as tf
 
@@ -77,6 +78,7 @@ def invalid_range(cursor, replay_capacity, stack_size, update_horizon):
        for i in range(stack_size + update_horizon)])
 
 
+@gin.configurable
 class OutOfGraphReplayBuffer(object):
   """A simple out-of-graph Replay Buffer.
 
@@ -105,6 +107,7 @@ class OutOfGraphReplayBuffer(object):
                max_sample_attempts=1000,
                extra_storage_types=None,
                observation_dtype=np.uint8,
+               terminal_dtype=np.uint8,
                action_shape=(),
                action_dtype=np.int32,
                reward_shape=(),
@@ -124,6 +127,8 @@ class OutOfGraphReplayBuffer(object):
         contents that will be stored and returned by sample_transition_batch.
       observation_dtype: np.dtype, type of the observations. Defaults to
         np.uint8 for Atari 2600.
+      terminal_dtype: np.dtype, type of the terminals. Defaults to np.uint8 for
+        Atari 2600.
       action_shape: tuple of ints, the shape for the action vector. Empty tuple
         means the action is a scalar.
       action_dtype: np.dtype, type of elements in the action.
@@ -140,16 +145,17 @@ class OutOfGraphReplayBuffer(object):
       raise ValueError('There is not enough capacity to cover '
                        'update_horizon and stack_size.')
 
-    tf.logging.info(
+    logging.info(
         'Creating a %s replay memory with the following parameters:',
         self.__class__.__name__)
-    tf.logging.info('\t observation_shape: %s', str(observation_shape))
-    tf.logging.info('\t observation_dtype: %s', str(observation_dtype))
-    tf.logging.info('\t stack_size: %d', stack_size)
-    tf.logging.info('\t replay_capacity: %d', replay_capacity)
-    tf.logging.info('\t batch_size: %d', batch_size)
-    tf.logging.info('\t update_horizon: %d', update_horizon)
-    tf.logging.info('\t gamma: %f', gamma)
+    logging.info('\t observation_shape: %s', str(observation_shape))
+    logging.info('\t observation_dtype: %s', str(observation_dtype))
+    logging.info('\t terminal_dtype: %s', str(terminal_dtype))
+    logging.info('\t stack_size: %d', stack_size)
+    logging.info('\t replay_capacity: %d', replay_capacity)
+    logging.info('\t batch_size: %d', batch_size)
+    logging.info('\t update_horizon: %d', update_horizon)
+    logging.info('\t gamma: %f', gamma)
 
     self._action_shape = action_shape
     self._action_dtype = action_dtype
@@ -163,6 +169,7 @@ class OutOfGraphReplayBuffer(object):
     self._update_horizon = update_horizon
     self._gamma = gamma
     self._observation_dtype = observation_dtype
+    self._terminal_dtype = terminal_dtype
     self._max_sample_attempts = max_sample_attempts
     if extra_storage_types:
       self._extra_storage_types = extra_storage_types
@@ -210,7 +217,7 @@ class OutOfGraphReplayBuffer(object):
                       self._observation_dtype),
         ReplayElement('action', self._action_shape, self._action_dtype),
         ReplayElement('reward', self._reward_shape, self._reward_dtype),
-        ReplayElement('terminal', (), np.uint8)
+        ReplayElement('terminal', (), self._terminal_dtype)
     ]
 
     for extra_replay_element in self._extra_storage_types:
@@ -241,7 +248,7 @@ class OutOfGraphReplayBuffer(object):
       observation: np.array with shape observation_shape.
       action: int, the action in the transition.
       reward: float, the reward received in the transition.
-      terminal: A uint8 acting as a boolean indicating whether the transition
+      terminal: np.dtype, acts as a boolean indicating whether the transition
                 was terminal (1) or not (0).
       *args: extra contents with shapes and dtypes according to
         extra_storage_types.
@@ -260,17 +267,40 @@ class OutOfGraphReplayBuffer(object):
     Args:
       *args: All the elements in a transition.
     """
-    cursor = self.cursor()
+    self._check_args_length(*args)
+    transition = {e.name: args[idx]
+                  for idx, e in enumerate(self.get_add_args_signature())}
+    self._add_transition(transition)
 
-    arg_names = [e.name for e in self.get_add_args_signature()]
-    for arg_name, arg in zip(arg_names, args):
-      self._store[arg_name][cursor] = arg
+  def _add_transition(self, transition):
+    """Internal add method to add transition dictionary to storage arrays.
+
+    Args:
+      transition: The dictionary of names and values of the transition
+                  to add to the storage.
+    """
+    cursor = self.cursor()
+    for arg_name in transition:
+      self._store[arg_name][cursor] = transition[arg_name]
 
     self.add_count += 1
     self.invalid_range = invalid_range(
         self.cursor(), self._replay_capacity, self._stack_size,
         self._update_horizon)
 
+  def _check_args_length(self, *args):
+    """Check if args passed to the add method have the same length as storage.
+
+    Args:
+      *args: Args for elements used in storage.
+
+    Raises:
+      ValueError: If args have wrong length.
+    """
+    if len(args) != len(self.get_add_args_signature()):
+      raise ValueError('Add expects {} elements, received {}'.format(
+          len(self.get_add_args_signature()), len(args)))
+
   def _check_add_types(self, *args):
     """Checks if args passed to the add method match those of the storage.
 
@@ -280,9 +310,7 @@ class OutOfGraphReplayBuffer(object):
     Raises:
       ValueError: If args have wrong shape or dtype.
     """
-    if len(args) != len(self.get_add_args_signature()):
-      raise ValueError('Add expects {} elements, received {}'.format(
-          len(self.get_add_args_signature()), len(args)))
+    self._check_args_length(*args)
     for arg_element, store_element in zip(args, self.get_add_args_signature()):
       if isinstance(arg_element, np.ndarray):
         arg_shape = arg_element.shape
@@ -555,7 +583,7 @@ class OutOfGraphReplayBuffer(object):
                       self._action_dtype),
         ReplayElement('next_reward', (batch_size,) + self._reward_shape,
                       self._reward_dtype),
-        ReplayElement('terminal', (batch_size,), np.uint8),
+        ReplayElement('terminal', (batch_size,), self._terminal_dtype),
         ReplayElement('indices', (batch_size,), np.int32)
     ]
     for element in self._extra_storage_types:
@@ -594,14 +622,14 @@ class OutOfGraphReplayBuffer(object):
       iteration_number: int, iteration_number to use as a suffix in naming
         numpy checkpoint files.
     """
-    if not tf.gfile.Exists(checkpoint_dir):
+    if not tf.io.gfile.exists(checkpoint_dir):
       return
 
     checkpointable_elements = self._return_checkpointable_elements()
 
     for attr in checkpointable_elements:
       filename = self._generate_filename(checkpoint_dir, attr, iteration_number)
-      with tf.gfile.Open(filename, 'wb') as f:
+      with tf.io.gfile.GFile(filename, 'wb') as f:
         with gzip.GzipFile(fileobj=f) as outfile:
           # Checkpoint the np arrays in self._store with np.save instead of
           # pickling the dictionary is critical for file size and performance.
@@ -623,7 +651,7 @@ class OutOfGraphReplayBuffer(object):
         stale_filename = self._generate_filename(checkpoint_dir, attr,
                                                  stale_iteration_number)
         try:
-          tf.gfile.Remove(stale_filename)
+          tf.io.gfile.remove(stale_filename)
         except tf.errors.NotFoundError:
           pass
 
@@ -643,14 +671,14 @@ class OutOfGraphReplayBuffer(object):
     # loading a partially-specified (i.e. corrupted) replay buffer.
     for attr in save_elements:
       filename = self._generate_filename(checkpoint_dir, attr, suffix)
-      if not tf.gfile.Exists(filename):
+      if not tf.io.gfile.exists(filename):
         raise tf.errors.NotFoundError(None, None,
                                       'Missing file: {}'.format(filename))
     # If we've reached this point then we have verified that all expected files
     # are available.
     for attr in save_elements:
       filename = self._generate_filename(checkpoint_dir, attr, suffix)
-      with tf.gfile.Open(filename, 'rb') as f:
+      with tf.io.gfile.GFile(filename, 'rb') as f:
         with gzip.GzipFile(fileobj=f) as infile:
           if attr.startswith(STORE_FILENAME_PREFIX):
             array_name = attr[len(STORE_FILENAME_PREFIX):]
@@ -678,7 +706,7 @@ class WrappedReplayBuffer(object):
   def __init__(self,
                observation_shape,
                stack_size,
-               use_staging=True,
+               use_staging=False,
                replay_capacity=1000000,
                batch_size=32,
                update_horizon=1,
@@ -687,6 +715,7 @@ class WrappedReplayBuffer(object):
                max_sample_attempts=1000,
                extra_storage_types=None,
                observation_dtype=np.uint8,
+               terminal_dtype=np.uint8,
                action_shape=(),
                action_dtype=np.int32,
                reward_shape=(),
@@ -710,6 +739,8 @@ class WrappedReplayBuffer(object):
         contents that will be stored and returned by sample_transition_batch.
       observation_dtype: np.dtype, type of the observations. Defaults to
         np.uint8 for Atari 2600.
+      terminal_dtype: np.dtype, type of the terminals. Defaults to np.uint8 for
+        Atari 2600.
       action_shape: tuple of ints, the shape for the action vector. Empty tuple
         means the action is a scalar.
       action_dtype: np.dtype, type of elements in the action.
@@ -745,6 +776,7 @@ class WrappedReplayBuffer(object):
           gamma,
           max_sample_attempts,
           observation_dtype=observation_dtype,
+          terminal_dtype=terminal_dtype,
           extra_storage_types=extra_storage_types,
           action_shape=action_shape,
           action_dtype=action_dtype,
@@ -765,7 +797,7 @@ class WrappedReplayBuffer(object):
       observation: np.array with shape observation_shape.
       action: int, the action in the transition.
       reward: float, the reward received in the transition.
-      terminal: A uint8 acting as a boolean indicating whether the transition
+      terminal: np.dtype, acts as a boolean indicating whether the transition
                 was terminal (1) or not (0).
       *args: extra contents with shapes and dtypes according to
         extra_storage_types.
@@ -781,18 +813,16 @@ class WrappedReplayBuffer(object):
       use_staging: bool, when True it would use a staging area to prefetch
         the next sampling batch.
     """
+    if use_staging:
+      logging.warning('use_staging=True is no longer supported')
     with tf.name_scope('sample_replay'):
       with tf.device('/cpu:*'):
         transition_type = self.memory.get_transition_elements()
-        transition_tensors = tf.py_func(
+        transition_tensors = tf.numpy_function(
             self.memory.sample_transition_batch, [],
             [return_entry.type for return_entry in transition_type],
             name='replay_sample_py_func')
         self._set_transition_shape(transition_tensors, transition_type)
-        if use_staging:
-          transition_tensors = self._set_up_staging(transition_tensors)
-          self._set_transition_shape(transition_tensors, transition_type)
-
         # Unpack sample transition into member variables.
         self.unpack_transition(transition_tensors, transition_type)
 
@@ -821,25 +851,8 @@ class WrappedReplayBuffer(object):
       prefetched_transition: tuple of tf.Tensors with shape
         memory.get_transition_elements() that have been previously prefetched.
     """
-    transition_type = self.memory.get_transition_elements()
-
-    # Create the staging area in CPU.
-    prefetch_area = tf.contrib.staging.StagingArea(
-        [shape_with_type.type for shape_with_type in transition_type])
-
-    # Store prefetch op for tests, but keep it private -- users should not be
-    # calling _prefetch_batch.
-    self._prefetch_batch = prefetch_area.put(transition)
-    initial_prefetch = tf.cond(
-        tf.equal(prefetch_area.size(), 0),
-        lambda: prefetch_area.put(transition), tf.no_op)
-
-    # Every time a transition is sampled self.prefetch_batch will be
-    # called. If the staging area is empty, two put ops will be called.
-    with tf.control_dependencies([self._prefetch_batch, initial_prefetch]):
-      prefetched_transition = prefetch_area.get()
-
-    return prefetched_transition
+    del transition  # Unused.
+    raise NotImplementedError
 
   def unpack_transition(self, transition_tensors, transition_type):
     """Unpacks the given transition into member variables.
